package dev.shadowsoffire.apotheosis.adventure;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.google.common.collect.ImmutableSet;
import dev.shadowsoffire.apotheosis.Apotheosis;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import dev.shadowsoffire.apotheosis.adventure.AdventureConfig;
import dev.shadowsoffire.apotheosis.adventure.AdventureEvents;
import dev.shadowsoffire.apotheosis.adventure.affix.AffixRegistry;
import dev.shadowsoffire.apotheosis.adventure.affix.reforging.ReforgingRecipe;
import dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvagingRecipe;
import dev.shadowsoffire.apotheosis.adventure.boss.BossEvents;
import dev.shadowsoffire.apotheosis.adventure.boss.BossRegistry;
import dev.shadowsoffire.apotheosis.adventure.boss.Exclusion;
import dev.shadowsoffire.apotheosis.adventure.boss.MinibossRegistry;
import dev.shadowsoffire.apotheosis.adventure.client.AdventureModuleClient;
import dev.shadowsoffire.apotheosis.adventure.compat.AdventureTOPPlugin;
import dev.shadowsoffire.apotheosis.adventure.compat.AdventureTwilightCompat;
import dev.shadowsoffire.apotheosis.adventure.compat.GatewaysCompat;
import dev.shadowsoffire.apotheosis.adventure.gen.BlacklistModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixConvertLootModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixHookLootModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixLootModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixLootPoolEntry;
import dev.shadowsoffire.apotheosis.adventure.loot.AffixLootRegistry;
import dev.shadowsoffire.apotheosis.adventure.loot.GemLootModifier;
import dev.shadowsoffire.apotheosis.adventure.loot.GemLootPoolEntry;
import dev.shadowsoffire.apotheosis.adventure.loot.LootCategory;
import dev.shadowsoffire.apotheosis.adventure.loot.RarityRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.AddSocketsRecipe;
import dev.shadowsoffire.apotheosis.adventure.socket.SocketingRecipe;
import dev.shadowsoffire.apotheosis.adventure.socket.UnnamingRecipe;
import dev.shadowsoffire.apotheosis.adventure.socket.WithdrawalRecipe;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.GemRegistry;
import dev.shadowsoffire.apotheosis.adventure.socket.gem.bonus.GemBonus;
import dev.shadowsoffire.apotheosis.adventure.spawner.RogueSpawnerRegistry;
import dev.shadowsoffire.apotheosis.util.AffixItemIngredient;
import dev.shadowsoffire.apotheosis.util.GemIngredient;
import dev.shadowsoffire.apotheosis.util.NameHelper;
import dev.shadowsoffire.placebo.block_entity.TickingBlockEntityType;
import dev.shadowsoffire.placebo.config.Configuration;
import dev.shadowsoffire.placebo.loot.LootSystem;
import dev.shadowsoffire.placebo.recipe.RecipeHelper;
import dev.shadowsoffire.placebo.registry.RegistryEvent;
import dev.shadowsoffire.placebo.tabs.TabFillingRegistry;
import java.io.File;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.function.Supplier;
import net.minecraft.core.BlockPos;
import net.minecraft.core.Registry;
import net.minecraft.core.registries.BuiltInRegistries;
import net.minecraft.resources.ResourceLocation;
import net.minecraft.world.entity.ai.attributes.Attributes;
import net.minecraft.world.entity.ai.attributes.RangedAttribute;
import net.minecraft.world.item.Item;
import net.minecraft.world.item.crafting.Recipe;
import net.minecraft.world.item.crafting.RecipeSerializer;
import net.minecraft.world.level.block.Block;
import net.minecraft.world.level.block.entity.BlockEntityType;
import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.common.crafting.CraftingHelper;
import net.minecraftforge.common.crafting.IIngredientSerializer;
import net.minecraftforge.eventbus.api.SubscribeEvent;
import net.minecraftforge.fml.InterModComms;
import net.minecraftforge.fml.ModList;
import net.minecraftforge.fml.event.lifecycle.FMLClientSetupEvent;
import net.minecraftforge.fml.event.lifecycle.FMLCommonSetupEvent;
import net.minecraftforge.fml.event.lifecycle.InterModProcessEvent;
import net.minecraftforge.fml.javafmlmod.FMLJavaModLoadingContext;
import net.minecraftforge.fml.util.ObfuscationReflectionHelper;
import net.minecraftforge.registries.ForgeRegistries;
import net.minecraftforge.registries.RegisterEvent;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

public class AdventureModule {
  public static final Logger LOGGER = LogManager.getLogger("Apotheosis : Adventure");
  
  public static final boolean STAGES_LOADED = ModList.get().isLoaded("gamestages");
  
  static final Map<ResourceLocation, LootCategory> IMC_TYPE_OVERRIDES = new HashMap<>();
  
  public static final boolean DEBUG = false;
  
  public AdventureModule() {
    Adventure.bootstrap();
  }
  
  @SubscribeEvent
  public void preInit(Apotheosis.ApotheosisConstruction e) {
    ObfuscationReflectionHelper.setPrivateValue(RangedAttribute.class, Attributes.f_22284_, Double.valueOf(200.0D), "f_22308_");
    ObfuscationReflectionHelper.setPrivateValue(RangedAttribute.class, Attributes.f_22285_, Double.valueOf(100.0D), "f_22308_");
  }
  
  @SubscribeEvent
  public void init(FMLCommonSetupEvent e) {
    reload(null);
    MinecraftForge.EVENT_BUS.register(new AdventureEvents());
    MinecraftForge.EVENT_BUS.register(new BossEvents());
    MinecraftForge.EVENT_BUS.addListener(this::reload);
    RarityRegistry.INSTANCE.registerToBus();
    AffixRegistry.INSTANCE.registerToBus();
    GemRegistry.INSTANCE.registerToBus();
    AffixLootRegistry.INSTANCE.registerToBus();
    BossRegistry.INSTANCE.registerToBus();
    RogueSpawnerRegistry.INSTANCE.registerToBus();
    MinibossRegistry.INSTANCE.registerToBus();
    Apotheosis.HELPER.registerProvider(f -> {
          f.addRecipe((Recipe)new SocketingRecipe());
          f.addRecipe((Recipe)new WithdrawalRecipe());
          f.addRecipe((Recipe)new UnnamingRecipe());
        });
    e.enqueueWork(() -> {
          if (ModList.get().isLoaded("gateways"))
            GatewaysCompat.register(); 
          if (ModList.get().isLoaded("theoneprobe"))
            AdventureTOPPlugin.register(); 
          if (ModList.get().isLoaded("twilightforest"))
            AdventureTwilightCompat.register(); 
          LootSystem.defaultBlockTable((Block)Adventure.Blocks.SIMPLE_REFORGING_TABLE.get());
          LootSystem.defaultBlockTable((Block)Adventure.Blocks.REFORGING_TABLE.get());
          LootSystem.defaultBlockTable((Block)Adventure.Blocks.SALVAGING_TABLE.get());
          LootSystem.defaultBlockTable((Block)Adventure.Blocks.GEM_CUTTING_TABLE.get());
          LootSystem.defaultBlockTable((Block)Adventure.Blocks.AUGMENTING_TABLE.get());
          Registry.m_122965_(BuiltInRegistries.f_257035_, new ResourceLocation("apotheosis", "random_affix_item"), AffixLootPoolEntry.TYPE);
          Registry.m_122965_(BuiltInRegistries.f_257035_, new ResourceLocation("apotheosis", "random_gem"), GemLootPoolEntry.TYPE);
          Exclusion.initSerializers();
          GemBonus.initCodecs();
          CraftingHelper.register(Apotheosis.loc("affix_item"), (IIngredientSerializer)AffixItemIngredient.Serializer.INSTANCE);
          CraftingHelper.register(Apotheosis.loc("gem"), (IIngredientSerializer)GemIngredient.Serializer.INSTANCE);
          TabFillingRegistry.register(Adventure.Tabs.ADVENTURE.getKey(), new Supplier[] { 
                (Supplier)Adventure.Items.COMMON_MATERIAL, (Supplier)Adventure.Items.UNCOMMON_MATERIAL, (Supplier)Adventure.Items.RARE_MATERIAL, (Supplier)Adventure.Items.EPIC_MATERIAL, (Supplier)Adventure.Items.MYTHIC_MATERIAL, (Supplier)Adventure.Items.GEM_DUST, (Supplier)Adventure.Items.GEM_FUSED_SLATE, (Supplier)Adventure.Items.SIGIL_OF_SOCKETING, (Supplier)Adventure.Items.SIGIL_OF_WITHDRAWAL, (Supplier)Adventure.Items.SIGIL_OF_REBIRTH, 
                (Supplier)Adventure.Items.SIGIL_OF_ENHANCEMENT, (Supplier)Adventure.Items.SIGIL_OF_UNNAMING, (Supplier)Adventure.Items.BOSS_SUMMONER, (Supplier)Adventure.Items.SALVAGING_TABLE, (Supplier)Adventure.Items.GEM_CUTTING_TABLE, (Supplier)Adventure.Items.SIMPLE_REFORGING_TABLE, (Supplier)Adventure.Items.REFORGING_TABLE, (Supplier)Adventure.Items.AUGMENTING_TABLE });
          TabFillingRegistry.register(Adventure.Tabs.ADVENTURE.getKey(), new Supplier[] { (Supplier)Adventure.Items.GEM });
        });
  }
  
  @SubscribeEvent
  public void tiles(RegistryEvent.Register<BlockEntityType<?>> e) {
    e.getRegistry().register(new TickingBlockEntityType(dev.shadowsoffire.apotheosis.adventure.boss.BossSpawnerBlock.BossSpawnerTile::new, ImmutableSet.of((Block)Adventure.Blocks.BOSS_SPAWNER.get()), false, true), "boss_spawner");
    e.getRegistry().register(new TickingBlockEntityType(dev.shadowsoffire.apotheosis.adventure.affix.reforging.ReforgingTableTile::new, ImmutableSet.of((Block)Adventure.Blocks.SIMPLE_REFORGING_TABLE.get(), (Block)Adventure.Blocks.REFORGING_TABLE.get()), true, false), "reforging_table");
    e.getRegistry().register(new BlockEntityType(dev.shadowsoffire.apotheosis.adventure.affix.salvaging.SalvagingTableTile::new, ImmutableSet.of((Block)Adventure.Blocks.SALVAGING_TABLE.get()), null), "salvaging_table");
    e.getRegistry().register(new TickingBlockEntityType(dev.shadowsoffire.apotheosis.adventure.affix.augmenting.AugmentingTableTile::new, ImmutableSet.of((Block)Adventure.Blocks.AUGMENTING_TABLE.get()), true, false), "augmenting_table");
  }
  
  @SubscribeEvent
  public void serializers(RegistryEvent.Register<RecipeSerializer<?>> e) {
    e.getRegistry().register(SocketingRecipe.Serializer.INSTANCE, "socketing");
    e.getRegistry().register(WithdrawalRecipe.Serializer.INSTANCE, "widthdrawal");
    e.getRegistry().register(UnnamingRecipe.Serializer.INSTANCE, "unnaming");
    e.getRegistry().register(AddSocketsRecipe.Serializer.INSTANCE, "add_sockets");
    e.getRegistry().register(SalvagingRecipe.Serializer.INSTANCE, "salvaging");
    e.getRegistry().register(ReforgingRecipe.Serializer.INSTANCE, "reforging");
  }
  
  @SubscribeEvent
  public void miscRegistration(RegisterEvent e) {
    if (e.getForgeRegistry() == ForgeRegistries.GLOBAL_LOOT_MODIFIER_SERIALIZERS.get()) {
      e.getForgeRegistry().register("gems", GemLootModifier.CODEC);
      e.getForgeRegistry().register("affix_loot", AffixLootModifier.CODEC);
      e.getForgeRegistry().register("affix_conversion", AffixConvertLootModifier.CODEC);
      e.getForgeRegistry().register("affix_hook", AffixHookLootModifier.CODEC);
    } 
    if (e.getForgeRegistry() == ForgeRegistries.BIOME_MODIFIER_SERIALIZERS.get())
      e.getForgeRegistry().register("blacklist", BlacklistModifier.CODEC); 
  }
  
  @SubscribeEvent
  public void client(FMLClientSetupEvent e) {
    e.enqueueWork(AdventureModuleClient::init);
    FMLJavaModLoadingContext.get().getModEventBus().register(new AdventureModuleClient());
  }
  
  @SubscribeEvent
  public void imc(InterModProcessEvent e) {
    e.getIMCStream().forEach(msg -> {
          switch (msg.method().toLowerCase(Locale.ROOT)) {
            case "loot_category_override":
              try {
                Map.Entry<Item, String> categoryOverride = msg.messageSupplier().get();
                ResourceLocation item = BuiltInRegistries.f_257033_.m_7981_(categoryOverride.getKey());
                LootCategory cat = LootCategory.byId(categoryOverride.getValue());
                if (cat == null)
                  throw new NullPointerException("Invalid loot category ID: " + (String)categoryOverride.getValue()); 
                IMC_TYPE_OVERRIDES.put(item, cat);
                LOGGER.info("Mod {} has overriden the loot category of {} to {}.", msg.senderModId(), item, cat.getName());
              } catch (Exception ex) {
                LOGGER.error(ex.getMessage());
                ex.printStackTrace();
              } 
              return;
          } 
          LOGGER.error("Unknown or invalid IMC Message: {}", msg);
        });
  }
  
  public void reload(Apotheosis.ApotheosisReloadEvent e) {
    Configuration mainConfig = new Configuration(new File(Apotheosis.configDir, "adventure.cfg"));
    Configuration nameConfig = new Configuration(new File(Apotheosis.configDir, "names.cfg"));
    AdventureConfig.load(mainConfig);
    NameHelper.load(nameConfig);
    if (e == null && mainConfig.hasChanged())
      mainConfig.save(); 
    if (e == null && nameConfig.hasChanged())
      nameConfig.save(); 
  }
  
  public static void debugLog(BlockPos pos, String name) {}
}
